-- You can add your own plugins here or in other files in this directory!
--  I promise not to create any merge conflicts in this directory :)
--
-- See the kickstart.nvim README for more information
return {
  {
    'joaodubas/gitlinker.nvim',
    config = function()
      local actions = require 'gitlinker.actions'
      local hosts = require 'gitlinker.hosts'
      require('gitlinker').setup {
        opts = {
          remote = 'origin',
          add_current_line_on_normal_mode = true,
          action_callback = actions.copy_to_clipboard,
          print_url = true,
        },
        callbacks = {
          ['github.com'] = hosts.get_github_type_url,
          ['bitbucket.org'] = hosts.get_bitbucket_type_url,
          ['gitea.dubas.dev'] = hosts.get_gitea_type_url,
        },
        mappings = '<leader>gy',
      }
    end,
  },
  {
    'NeogitOrg/neogit',
    dependencies = {
      'nvim-lua/plenary.nvim',
      'sindrets/diffview.nvim',
      'nvim-telescope/telescope.nvim',
    },
    opts = {
      git_services = {
        ['gitea.dubas.dev'] = 'https://gitea.dubas.dev/${owner}/${repository}/compare/${branch_name}',
      },
    },
    keys = {
      { '<leader>gs', '<cmd>Neogit<cr>', desc = 'Git status' },
    },
  },
  'nvim-treesitter/nvim-treesitter-context',
  {
    'kevinhwang91/nvim-ufo',
    dependencies = { 'kevinhwang91/promise-async' },
    event = 'BufRead',
    keys = function()
      local ufo = require 'ufo'
      return {
        { 'zR', ufo.openAllFolds, { desc = 'Open all folds' } },
        { 'zM', ufo.closeAllFolds, { desc = 'Close all folds' } },
        { 'zr', ufo.openFoldsExceptKinds, { desc = 'Open fold' } },
        { 'zm', ufo.closeFoldsWith, { desc = 'Close fold' } },
      }
    end,
    opts = function()
      local handler = function(virtText, lnum, endLnum, width, truncate)
        local newVirtText = {}
        local suffix = (' 󰁂 %d '):format(endLnum - lnum)
        local sufWidth = vim.fn.strdisplaywidth(suffix)
        local targetWidth = width - sufWidth
        local curWidth = 0
        for _, chunk in ipairs(virtText) do
          local chunkText = chunk[1]
          local chunkWidth = vim.fn.strdisplaywidth(chunkText)
          if targetWidth > curWidth + chunkWidth then
            table.insert(newVirtText, chunk)
          else
            chunkText = truncate(chunkText, targetWidth - curWidth)
            local hlGroup = chunk[2]
            table.insert(newVirtText, { chunkText, hlGroup })
            chunkWidth = vim.fn.strdisplaywidth(chunkText)
            -- str width returned from truncate() may less than 2nd argument, need padding
            if curWidth + chunkWidth < targetWidth then
              suffix = suffix .. (' '):rep(targetWidth - curWidth - chunkWidth)
            end
            break
          end
          curWidth = curWidth + chunkWidth
        end
        table.insert(newVirtText, { suffix, 'MoreMsg' })
        return newVirtText
      end

      return {
        fold_virt_text_handler = handler,
        provider_selector = function(_, _, _)
          return { 'treesitter', 'indent' }
        end,
      }
    end,
  },
  {
    'stevearc/oil.nvim',
    dependencies = { 'nvim-tree/nvim-web-devicons' },
    keys = {
      { '-', '<cmd>Oil<cr>', desc = 'Open parent directory' },
    },
    opts = {},
  },
  {
    'vhyrro/luarocks.nvim',
    priority = 1000,
    config = true,
  },
  {
    'EvWilson/slimux.nvim',
    lazy = true,
    opts = function()
      local status_ok, slimux = pcall(require, 'slimux')
      if not status_ok then
        return {}
      end
      return {
        target_socket = slimux.get_tmux_socket(),
        target_pane = string.format('%s.1', slimux.get_tmux_window()),
      }
    end,
    keys = function()
      local status_ok, which_key = pcall(require, 'which-key')
      if status_ok then
        which_key.add {
          { '<leader>m', group = 'Toggle ter[m]inal' },
        }
      end
      local slimux_status_ok, slimux = pcall(require, 'slimux')
      if not slimux_status_ok then
        return {}
      end
      return {
        {
          '<leader>xr',
          slimux.send_highlighted_text,
          mode = 'v',
          desc = 'Send currently highlighted text to configured tmux pane',
        },
        {
          '<leader>r',
          slimux.send_paragraph_text,
          mode = 'n',
          desc = 'Send paragraph under cursor to configured tmux pane',
        },
      }
    end,
  },
  {
    'ravitemer/mcphub.nvim',
    dependencies = {
      'nvim-lua/plenary.nvim',
    },
    cmd = 'MCPHub',
    build = 'bundled_build.lua',
    opts = {
      port = 37373,
      config = vim.fn.expand '~/.config/mcphub/servers.json',
      native_servers = {},
      auto_approve = true,
      auto_toggle_mcp_servers = true,
      extensions = {},
      ui = {
        window = {
          width = 0.8,
          height = 0.8,
          relative = 'editor',
          zindex = 50,
          border = 'rounded',
        },
        wo = {},
      },
      use_bundled_binary = true,
      log = {
        level = vim.log.levels.WARN,
        prefix = '[MCPHub]',
      },
    },
  },
  {
    'Davidyz/VectorCode',
    dependencies = {
      'nvim-lua/plenary.nvim',
    },
    version = vim.fn.system { 'mise', 'current', 'pipx:vectorcode' },
    opts = function()
      return {
        async_opts = {
          debounce = 10,
          events = { 'BufWritePost', 'InsertEnter', 'BufReadPost' },
          exclude_this = true,
          n_query = 1,
          notify = false,
          query_cb = require('vectorcode.utils').make_surrounding_lines_cb(-1),
          run_on_register = false,
        },
        async_backend = 'default',
        exclude_this = true,
        n_query = 1,
        notify = true,
        timeout_ms = 5000,
        on_setup = {
          update = false,
          lsp = false,
        },
        sync_log_env_var = false,
      }
    end,
  },
  {
    'olimorris/codecompanion.nvim',
    dependencies = {
      'nvim-lua/plenary.nvim',
      'nvim-treesitter/nvim-treesitter',
      {
        'MeanderingProgrammer/render-markdown.nvim',
        opts = {
          file_types = { 'markdown', 'codecompanion' },
        },
        ft = { 'markdown', 'codecompanion' },
      },
    },
    opts = function()
      return {
        display = {
          action_palette = {
            width = 95,
            height = 15,
            prompt = 'Prompt',
            provide = 'telescope',
          },
        },
        adapters = {
          gemini_pro = function()
            return require('codecompanion.adapters').extend('gemini', {
              name = 'gemini_pro',
              schema = {
                model = {
                  default = 'gemini-2.5-pro-exp-03-25',
                },
              },
            })
          end,
          gemini_flash = function()
            return require('codecompanion.adapters').extend('gemini', {
              name = 'gemini_flash',
              schema = {
                model = {
                  default = 'gemini-2.0-flash',
                },
              },
            })
          end,
        },
        extensions = {
          vectorcode = {
            opts = {
              add_tool = true,
              add_slash_command = true,
              tool_opts = {},
            },
          },
          mcphub = {
            callback = 'mcphub.extensions.codecompanion',
            opts = {
              make_slash_commands = true,
              make_vars = true,
              show_result_in_chat = true,
            },
          },
        },
        strategies = {
          chat = { adapter = 'gemini_pro' },
          inline = { adapter = 'gemini_flash' },
        },
      }
    end,
    keys = {
      {
        '<leader>aa',
        '<cmd>CodeCompanionActions<cr>',
        mode = { 'n', 'v' },
        desc = 'CodeCompaion actions',
        noremap = true,
        silent = true,
      },
      {
        '<leader>at',
        '<cmd>CodeCompanionChat Toggle<cr>',
        mode = { 'n', 'v' },
        desc = 'CodeCompanion toggle chat',
        noremap = true,
        silent = true,
      },
      {
        'ga',
        '<cmd>CodeCompanionChat add<cr>',
        mode = 'v',
        desc = 'CodeCompanion add to chat',
        noremap = true,
        silent = true,
      },
    },
  },
  {
    'supermaven-inc/supermaven-nvim',
    opts = {
      keymaps = {
        accept_suggestion = '<Tab>',
        clear_suggestion = '<C-]>',
        accept_word = '<C-j>',
      },
    },
  },
  {
    'sphamba/smear-cursor.nvim',
    opts = {
      cursor_color = '#ff8800',
      gamma = 1,
      hide_target_hack = true,
      stiffness = 0.3,
      trailing_exponent = 10,
      trailing_stiffness = 0.1,
    },
  },
}
